(function (window, $) {

    setDefaultPassword();

    var arrayPassword = {};

    $(document).ready(function () {
        /* Remove in version 1.0.9 */
        $("#post_password").change(function() {
            if ( $(this).val().indexOf(" ") !== -1 ) {
                toastr.error('Please remove spaces in password!', 'Password Protect WordPress');
                $('#publish').prop( "disabled", true );
            } else {
                $('#publish').prop( "disabled", false );
            }
        });

        $("input[type=radio][name=visibility]").change(function() {
            if ( this.value !== "password" ) {
                $('#publish').prop( "disabled", false );
                $("#post_password").val('');
            }
        });
        /* End remove in version 1.0.9 */

        $('.pda-selected-role-select2').on('focus', function () {
            var role = this.value;
            // Add Value to MAP
            arrayPassword[role] = $('#post-protection-password').val();
            $('#' + role).val($('#post-protection-password').val());
        }).change(function () {
            var role = $('.pda-selected-role-select2').val();

            var value;
            if (typeof (arrayPassword[role]) !== 'undefined') {
                value = arrayPassword[role];
            } else {
                value = $('#post-protection-password-' + role).text() === "" ? "" : $('#post-protection-password-' + role).text();
            }

            if (role === 'global') {
                $('#post-protection-password').hide();
                $('#ppwp_multiple_password').show();
            } else {
                $('#post-protection-password').show();
                $('#ppwp_multiple_password').hide();
            }

            $('#post-protection-password').val(value);

            if ($("#is_role_selected").val() === 'global') {
                $("#label-password-post").text("Passwords")
            } else {
                $("#label-password-post").text("Password")
            }
        });

        $('.pda-selected-role-select2').trigger('change');

    });

    $('.edit-post-protection').click(function () {
        $('#post-protection').show();
        $(this).hide();
    });

    $('.button-cancel').click(function () {
        $('#post-protection').hide();
        $('.edit-post-protection').show();
    });

    function setDefaultPassword() {
        var role = $('.pda-selected-role-select2').val();
        if (role) {
            var value = $('#post-protection-password-' + role).text() === "" ? "" : $('#post-protection-password-' + role).text();
            $('#post-protection-password').val(value);
        }
    }

    function validateBeforeSubmit() {
        if ($("#is_role_selected").val() === 'global') {
            const passwords = $(".ppwp_multiple_password").val().split("\n");
            function checkPasswordNoSpace(password) {
                return password.indexOf(" ") === -1
            }
            if ((new Set(passwords)).size !== passwords.length) {
                toastr.error('You can\'t create duplicate password. Please try again.', 'Password Protect WordPress');
                return false;
            }
            if (!passwords.every(checkPasswordNoSpace)) {
                toastr.error('Spaces not accepted in password. Please remove them and try again.', 'Password Protect WordPress');
                return false;
            }
        } else {
            if ($("#post-protection-password").val().indexOf(" ") !== -1) {
                toastr.error('Spaces not accepted in password. Please remove them and try again.', 'Password Protect WordPress');
                return false;
            }
        }

        return true;
    }

    $("#save_password").click(function (evt) {
        evt.preventDefault();

        if ( ! validateBeforeSubmit() ) {
            return;
        }

        savePassword({
            save_password: $(".post-protection-password").val(),
            is_role_selected: $("#is_role_selected").val(),
            id_page_post: $("#id_page_post").val(),
            ppwp_multiple_password: $("#ppwp_multiple_password").val(),
        }, function (error) {
            if (error) {
                console.error(error);
                toastr.error('Passwords updated fail!', 'Password Protect WordPress');
            }
        });
    });

    function savePassword(settings, cb) {
        var _data = {
            action: 'wp_protect_password_save_password',
            settings: settings,
        }
        $('#save_password').text('Submitting');
        $('#save_password').prop("disabled", true);
        $.ajax({
            url: save_password_data.ajax_url,
            type: 'POST',
            data: _data,
            success: function (data) {
                $('#save_password').text('Submit');
                $('#save_password').prop("disabled", false);
                if (data) {
                    if (data['is_error']) {
                        toastr.error(data['message'], 'Password Protect WordPress');
                    } else {
                        changeValueRoles(data);
                        toastr.success('Passwords updated successfully!', 'Password Protect WordPress');
                    }
                } else {
                    toastr.error('Passwords updated fail!', 'Password Protect WordPress');
                    console.log("Failed", data);
                }
                cb();
            },
            error: function (error) {
                toastr.error('Passwords updated fail!', 'Password Protect WordPress');
                console.log("Error", error);
                cb(error);
            },
            timeout: 5000
        });
    }

    function changeValueRoles(data) {
        var rolesFiltered = Object.keys(data)
            .filter(function (role) {
                return '' !== data[role];
            });
        var roles = rolesFiltered.map(function (role) {
            return "<span>" + role + "</span>";
        })
            .join(' ');
        var rolesText = rolesFiltered.length > 1 ? rolesFiltered.length + ' roles' : rolesFiltered.length + ' role';
        $("#all_roles_select").html(roles);
        $("#number_roles").text(rolesText);
    }

})(window, jQuery);
