<?php

/**
 *
 * @link              www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since             1.1.2.1
 * @package           Wp_Protect_Password
 *
 * @wordpress-plugin
 * Plugin Name:       Password Protect WordPress Lite
 * Plugin URI:        https://preventdirectaccess.com/extensions/password-protect-wordpress-plugin/?utm_source=user-website&utm_medium=pluginsite_link&utm_campaign=password-protect-wp
 * Description:       Password protect WordPress pages or posts by user roles. You can set different passwords for different user roles.
 * Version:           1.1.2.1
 * Author:            BWPS
 * Author URI:        https://preventdirectaccess.com/password-protect-wordpress-plugin/?utm_source=wp.org&utm_medium=post&utm_campaign=plugin-link
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       password-protect-wp-page
 * Domain Path:       /languages
 */

define( 'PDA_PWD_DIR_PATH', plugin_dir_path( __FILE__ ) );
define( 'PDA_ADMIN_GOLD_PATH', PDA_PWD_DIR_PATH . 'admin/gold/' );
if ( is_dir( PDA_ADMIN_GOLD_PATH ) ) {
	define( 'PROTECT_BY_PASSWORD', '1.0.8' );
} else {
	define( 'PROTECT_BY_PASSWORD', '1.1.2' );
}

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'WPPP_BASE_FILE',  __FILE__ );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-wp-protect-password-activator.php
 */
function activate_wp_protect_password() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp-protect-password-activator.php';
	Wp_Protect_Password_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-wp-protect-password-deactivator.php
 */
function deactivate_wp_protect_password() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp-protect-password-deactivator.php';
	Wp_Protect_Password_Deactivator::deactivate();
}

/**
 * Handle remove data when uninstall plugin
 */
function wp_protect_password_uninstall() {
	delete_pda_password_options();
}



/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-wp-protect-password.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_wp_protect_password() {

	$plugin = new Wp_Protect_Password();
	$plugin->run();
	register_activation_hook( __FILE__, 'activate_wp_protect_password' );
	register_deactivation_hook( __FILE__, 'deactivate_wp_protect_password' );
	if ( is_gold_function() ) {
		Puc_v4_Factory::buildUpdateChecker(
			'https://s3-ap-southeast-1.amazonaws.com/bwps.gold.plugins/wp_protect_password/metadata.json',
			__FILE__,
			'wp_protect_password'
		);
	}
	register_uninstall_hook( __FILE__, 'wp_protect_password_uninstall' );

}

run_wp_protect_password();
